"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _uuid = _interopRequireDefault(require("uuid"));

var _componentWrap = _interopRequireDefault(require("./componentWrap"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// 对react 渲染进行优化
var reactOptimizeModelPlugin = // 用来标识模型插件身份
function reactOptimizeModelPlugin() {
  var _this = this;

  _classCallCheck(this, reactOptimizeModelPlugin);

  _defineProperty(this, "schemaInstance", void 0);

  _defineProperty(this, "mApi", void 0);

  _defineProperty(this, "onComponentsWrap", function (components) {
    var wrappedComps = {};
    Object.keys(components).forEach(function (key) {
      var comp = components[key];
      wrappedComps[key] = (0, _componentWrap["default"])(comp);
    });
    return wrappedComps;
  });

  _defineProperty(this, "onSchemaInstantiated", function (params) {
    var event = params.event;
    var schemaInstance = event.schemaInstance; // 将实例保存起来

    _this.schemaInstance = schemaInstance;

    _this.schemaInstance.loopDown(function (node) {
      if (!node.attributes) return; // 对所有节点进行一次初始标记

      node.attributes.__renderOptimizeMark__ = "sifo-react-optimize-init-mark"; // eslint-disable-line
      // 禁用渲染优化标记初始为false

      node.attributes.muteRenderOptimizeMark = false; // eslint-disable-line
    });
  });

  _defineProperty(this, "onModelApiCreated", function (params) {
    var mApi = params.mApi,
        event = params.event;
    var applyModelApiMiddleware = event.applyModelApiMiddleware;

    var setAttributesMiddleware = function setAttributesMiddleware(next) {
      return function (key) {
        var guid = (0, _uuid["default"])(); // 进行渲染标识

        _this.schemaInstance.loopUp(function (node) {
          if (!node.attributes) return;
          node.attributes.__renderOptimizeMark__ = guid; // eslint-disable-line
        }, key); // 从此key向上标记，包含此key的节点


        for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        return next.apply(void 0, [key].concat(args));
      };
    }; // 对mApi的setAttributes方法进行再组装


    applyModelApiMiddleware('setAttributes', setAttributesMiddleware); // 替换渲染组件方法

    var replaceComponentMiddleware = function replaceComponentMiddleware(next) {
      return function (key, componentName) {
        mApi.setAttributes(key, {}, false); // 强制标识

        return next(key, componentName);
      };
    };

    applyModelApiMiddleware('replaceComponent', replaceComponentMiddleware); // 强制渲染

    var forceRefreshMiddleware = function forceRefreshMiddleware(next) {
      return function () {
        var guid = (0, _uuid["default"])(); // 进行渲染标识

        _this.schemaInstance.loopDown(function (node) {
          if (!node.attributes) return;
          node.attributes.__renderOptimizeMark__ = guid; // eslint-disable-line
        }); // 从此key向下标记，包含此key的节点


        next.apply(void 0, arguments);
        return mApi.refresh();
      };
    };

    applyModelApiMiddleware('forceRefresh', forceRefreshMiddleware);
  });

  this.schemaInstance = null;
  this.mApi = null;
} // 带优化的SCU组件
;

_defineProperty(reactOptimizeModelPlugin, "ID", 'sifo-react-optimize-model-plugin');

var _default = reactOptimizeModelPlugin;
exports["default"] = _default;